@@ -249,16 +249,18 @@ bool Application::parseCommandLineArgs() {
         }
         else {
             if(!parser.isSet(desktopOption) && !parser.isSet(desktopOffOption)) {
+                bool reopenLastTabs = false;
                 QStringList paths = parser.positionalArguments();
                 if(paths.isEmpty()) {
                     // if no path is specified and we're using daemon mode,
                     // don't open current working directory
                     if(!daemonMode_) {
+                        reopenLastTabs = true; // open last tab paths only if no path is specified
                         paths.push_back(QDir::currentPath());
                     }
                 }
                 if(!paths.isEmpty()) {
-                    launchFiles(QDir::currentPath(), paths, parser.isSet(newWindowOption));
+                    launchFiles(QDir::currentPath(), paths, parser.isSet(newWindowOption), reopenLastTabs);
                 }
                 keepRunning = true;
             }
@@ -293,11 +295,13 @@ bool Application::parseCommandLineArgs() {
         }
         else {
             if(!parser.isSet(desktopOption) && !parser.isSet(desktopOffOption)) {
+                bool reopenLastTabs = false;
                 QStringList paths = parser.positionalArguments();
                 if(paths.isEmpty()) {
+                    reopenLastTabs = true; // open last tab paths only if no path is specified
                     paths.push_back(QDir::currentPath());
                 }
-                iface.call(QStringLiteral("launchFiles"), QDir::currentPath(), paths, parser.isSet(newWindowOption));
+                iface.call(QStringLiteral("launchFiles"), QDir::currentPath(), paths, parser.isSet(newWindowOption), reopenLastTabs);
             }
         }
     }
@@ -505,10 +509,18 @@ void Application::connectToServer() {
     dlg->show();
 }
 
-void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow) {
+void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow, bool reopenLastTabs) {
     Fm::FilePathList pathList;
     Fm::FilePath cwd_path;
-    QStringList::iterator it;
+
+    reopenLastTabs = reopenLastTabs && settings_.reopenLastTabs() && !settings_.tabPaths().isEmpty();
+    if(reopenLastTabs) {
+        paths = settings_.tabPaths();
+        paths.removeDuplicates();
+        // forget tab paths with next windows until the last one is closed
+        settings_.setTabPaths(QStringList());
+    }
+
     for(const QString& it : qAsConst(paths)) {
         QByteArray pathName = it.toLocal8Bit();
         Fm::FilePath path;
@@ -550,6 +562,24 @@ void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow)
     else {
         Launcher(nullptr).launchPaths(nullptr, pathList);
     }
+
+    // if none of the last tabs can be opened and there is no main window yet,
+    // open the current directory
+    if(reopenLastTabs) {
+        bool hasWindow = false;
+        const QWidgetList windows = topLevelWidgets();
+        for(const auto& win : windows) {
+            if(win->inherits("PCManFM::MainWindow")) {
+                hasWindow = true;
+                break;
+            }
+        }
+        if(!hasWindow) {
+            paths.clear();
+            paths.push_back(QDir::currentPath());
+            launchFiles(QDir::currentPath(), paths, inNewWindow, false);
+        }
+    }
 }
 
 void Application::openFolders(Fm::FileInfoList files) {
@@ -636,7 +666,6 @@ DesktopWindow* Application::createDesktopWindow(int screenNum) {
     }
     else {
         QRect rect;
-        const auto allScreens = screens();
         if(auto screen = window->getDesktopScreen()) {
             rect = screen->geometry();
         }
@@ -684,7 +713,40 @@ void Application::editBookmarks() {
 }
 
 void Application::initVolumeManager() {
+    // WARNING: If the user tries to mount a volume before auto-mounting is started —
+    // and that is especially possible with an encrypted volume — a complete freeze might
+    // happen because MountOperation::wait() uses a local QEventLoop. Therefore, we need
+    // to start auto-mounting only after all probable GUI mount operations are finished.
+    QList<Fm::MountOperation*> ops;
+    const auto windows = topLevelWidgets();
+    for(const auto& win : windows) {
+        if(win->inherits("PCManFM::MainWindow")) {
+            MainWindow* mainWindow = static_cast<MainWindow*>(win);
+            ops << mainWindow->pendingMountOperations();
+        }
+    }
+    if(ops.isEmpty()) { // no pending mount operation
+        reallyInitVolumeManager();
+        return;
+    }
+    for(const Fm::MountOperation* op : ops) {
+        connect(op, &QObject::destroyed, this, [this] {
+            const auto windows = topLevelWidgets();
+            for(const auto& win : windows) {
+                if(win->inherits("PCManFM::MainWindow")) {
+                    MainWindow* mainWindow = static_cast<MainWindow*>(win);
+                    if(!mainWindow->pendingMountOperations().isEmpty()) {
+                        return;
+                    }
+                }
+            }
+            // now, there is no pending mount operation
+            reallyInitVolumeManager();
+        });
+    }
+}
 
+void Application::reallyInitVolumeManager() {
     g_signal_connect(volumeMonitor_, "volume-added", G_CALLBACK(onVolumeAdded), this);
 
     if(settings_.mountOnStartup()) {
