@@ -73,7 +73,7 @@ public:
     }
 
     // public interface exported via dbus
-    void launchFiles(QString cwd, QStringList paths, bool inNewWindow);
+    void launchFiles(QString cwd, QStringList paths, bool inNewWindow, bool reopenLastTabs);
     void setWallpaper(QString path, QString modeString);
     void preferences(QString page);
     void desktopPrefrences(QString page);
@@ -128,6 +128,7 @@ private Q_SLOTS:
 private:
     void initWatch();
     void installSigtermHandler();
+    void reallyInitVolumeManager();
 
     bool isPrimaryInstance;
     Fm::LibFmQt libFm_;
