@@ -40,6 +40,7 @@
 #include <QStandardPaths>
 #include <QClipboard>
 #include <QWindow>
+#include <QRandomGenerator>
 
 #include "./application.h"
 #include "mainwindow.h"
@@ -135,7 +136,6 @@ DesktopWindow::DesktopWindow(int screenNum):
         connect(proxyModel_, &Fm::ProxyFolderModel::rowsAboutToBeRemoved, this, &DesktopWindow::onRowsAboutToBeRemoved);
         connect(proxyModel_, &Fm::ProxyFolderModel::layoutChanged, this, &DesktopWindow::onLayoutChanged);
         connect(proxyModel_, &Fm::ProxyFolderModel::sortFilterChanged, this, &DesktopWindow::onModelSortFilterChanged);
-        connect(proxyModel_, &Fm::ProxyFolderModel::dataChanged, this, &DesktopWindow::onDataChanged);
     }
 
     // remove frame
@@ -312,8 +312,15 @@ void DesktopWindow::createTrashShortcut(int items) {
 
 void DesktopWindow::createHomeShortcut() {
     GKeyFile* kf = g_key_file_new();
-    g_key_file_set_string(kf, "Desktop Entry", "Type", "Application");
-    g_key_file_set_string(kf, "Desktop Entry", "Exec", Fm::CStrPtr(g_strconcat("pcmanfm-qt ", Fm::FilePath::homeDir().toString().get(), nullptr)).get());
+    if(static_cast<Application* >(qApp)->settings().openWithDefaultFileManager()) {
+        // let the default file manager open the home folder
+        g_key_file_set_string(kf, "Desktop Entry", "Type", "Link");
+        g_key_file_set_string(kf, "Desktop Entry", "URL", Fm::FilePath::homeDir().toString().get());
+    }
+    else {
+        g_key_file_set_string(kf, "Desktop Entry", "Type", "Application");
+        g_key_file_set_string(kf, "Desktop Entry", "Exec", Fm::CStrPtr(g_strconcat("pcmanfm-qt ", Fm::FilePath::homeDir().toString().get(), nullptr)).get());
+    }
     g_key_file_set_string(kf, "Desktop Entry", "Icon", "user-home");
     g_key_file_set_string(kf, "Desktop Entry", "Name", g_get_user_name());
 
@@ -410,7 +417,7 @@ void DesktopWindow::updateTrashIcon() {
 
 bool DesktopWindow::isTrashCan(std::shared_ptr<const Fm::FileInfo> file) const {
     bool ret(false);
-    if(file && (file->isDesktopEntry() || file->isShortcut()) && trashMonitor_) {
+    if(file && file->isDesktopEntry() && trashMonitor_) {
         const QString fileName = QString::fromStdString(file->name());
         const char* execStr = fileName == QLatin1String("trash-can.desktop")
                                 ? "pcmanfm-qt trash:///" : nullptr;
@@ -512,27 +519,29 @@ void DesktopWindow::setWallpaperRandomize(bool randomize) {
 
 QImage DesktopWindow::getWallpaperImage() const {
     QImage image(wallpaperFile_);
-    Settings& settings = static_cast<Application* >(qApp)->settings();
-    if(settings.transformWallpaper()
-       && (wallpaperFile_.endsWith(QLatin1String(".jpg"), Qt::CaseInsensitive)
-           || wallpaperFile_.endsWith(QLatin1String(".jpeg"), Qt::CaseInsensitive))) {
-        // auto-transform jpeg images based on their EXIF data
-        QImageReader reader(wallpaperFile_);
-        QImageIOHandler::Transformations tr = reader.transformation();
-        QMatrix m;
-        // mirroring
-        if(tr & QImageIOHandler::TransformationMirror) {
-            m.scale(-1, 1);
-        }
-        if(tr & QImageIOHandler::TransformationFlip) {
-            m.scale(1, -1);
-        }
-        // rotation
-        if(tr & QImageIOHandler::TransformationRotate90) {
-            m.rotate(90);
-        }
-        if(!m.isIdentity()) {
-            image = image.transformed(m);
+    if(!image.isNull()) {
+        Settings& settings = static_cast<Application* >(qApp)->settings();
+        if(settings.transformWallpaper()
+        && (wallpaperFile_.endsWith(QLatin1String(".jpg"), Qt::CaseInsensitive)
+            || wallpaperFile_.endsWith(QLatin1String(".jpeg"), Qt::CaseInsensitive))) {
+            // auto-transform jpeg images based on their EXIF data
+            QImageReader reader(wallpaperFile_);
+            QImageIOHandler::Transformations tr = reader.transformation();
+            QMatrix m;
+            // mirroring
+            if(tr & QImageIOHandler::TransformationMirror) {
+                m.scale(-1, 1);
+            }
+            if(tr & QImageIOHandler::TransformationFlip) {
+                m.scale(1, -1);
+            }
+            // rotation
+            if(tr & QImageIOHandler::TransformationRotate90) {
+                m.rotate(90);
+            }
+            if(!m.isIdentity()) {
+                image = image.transformed(m);
+            }
         }
     }
     return image;
@@ -624,7 +633,7 @@ void DesktopWindow::updateWallpaper() {
         bool perScreenWallpaper(screen != nullptr && screen->virtualSiblings().size() > 1 && settings.perScreenWallpaper());
         if(wallpaperMode_ == WallpaperTile) { // use the original size
             image = getWallpaperImage();
-            if (!image.isNull()) {
+            if(!image.isNull()) {
                 // Note: We can't use the QPainter::drawTiledPixmap(), because it doesn't tile
                 // correctly for background pixmaps bigger than the current screen size.
                 const QSize s = size();
@@ -644,11 +653,13 @@ void DesktopWindow::updateWallpaper() {
                 QPainter painter{&pixmap};
                 pixmap.fill(bgColor_);
                 image = getWallpaperImage();
-                QImage scaled;
-                const auto screens = screen->virtualSiblings();
-                for(const auto& scr : screens) {
-                    scaled = image.scaled(scr->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-                    painter.drawImage(scr->geometry().x(), scr->geometry().y(), scaled);
+                if(!image.isNull()) {
+                    QImage scaled;
+                    const auto screens = screen->virtualSiblings();
+                    for(const auto& scr : screens) {
+                        scaled = image.scaled(scr->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+                        painter.drawImage(scr->geometry().x(), scr->geometry().y(), scaled);
+                    }
                 }
             }
             else {
@@ -663,60 +674,63 @@ void DesktopWindow::updateWallpaper() {
                 QPainter painter{&pixmap};
                 pixmap.fill(bgColor_);
                 image = getWallpaperImage();
-                QImage scaled;
-                int x, y;
-                const auto screens = screen->virtualSiblings();
-                if(wallpaperMode_ == WallpaperCenter) {
-                    for(const auto& scr : screens) {
-                        // get the gap between image and screen to avoid overlapping and displacement
-                        int x_gap = (image.width() - scr->geometry().width()) / 2;
-                        int y_gap = (image.height() - scr->geometry().height()) / 2;
-                        scaled = image.copy(qMax(x_gap, 0), qMax(y_gap, 0), scr->geometry().width(), scr->geometry().height());
-                        x = scr->geometry().x() + qMax(0, -x_gap);
-                        y = scr->geometry().y() + qMax(0, - y_gap);
-                        painter.drawImage(x, y, scaled);
+                if(!image.isNull()) {
+                    QImage scaled;
+                    int x, y;
+                    const auto screens = screen->virtualSiblings();
+                    if(wallpaperMode_ == WallpaperCenter) {
+                        for(const auto& scr : screens) {
+                            // get the gap between image and screen to avoid overlapping and displacement
+                            int x_gap = (image.width() - scr->geometry().width()) / 2;
+                            int y_gap = (image.height() - scr->geometry().height()) / 2;
+                            scaled = image.copy(qMax(x_gap, 0), qMax(y_gap, 0), scr->geometry().width(), scr->geometry().height());
+                            x = scr->geometry().x() + qMax(0, -x_gap);
+                            y = scr->geometry().y() + qMax(0, - y_gap);
+                            painter.drawImage(x, y, scaled);
+                        }
                     }
-                }
-                else if(wallpaperMode_ == WallpaperFit || wallpaperMode_ == WallpaperZoom) {
-                    for(const auto& scr : screens) {
-                        // get the screen-to-image ratio to calculate the scale factors
-                        const qreal w_ratio = static_cast<qreal>(scr->geometry().width()) / image.width();
-                        const qreal h_ratio = static_cast<qreal>(scr->geometry().height()) / image.height();
-                        if(w_ratio <= h_ratio) {
-                           if(wallpaperMode_ == WallpaperFit) {
-                                // fit horizontally
-                                scaled = image.scaledToWidth(scr->geometry().width(), Qt::SmoothTransformation);
-                                x = scr->geometry().x();
-                                y = scr->geometry().y() + (scr->geometry().height() - scaled.height()) / 2;
-                           }
-                           else { // zoom
-                                // fit vertically
-                                scaled = image.scaledToHeight(scr->geometry().height(), Qt::SmoothTransformation);
-                                // crop to avoid overlapping
-                                int x_gap = (scaled.width() - scr->geometry().width()) / 2;
-                                scaled = scaled.copy(x_gap, 0, scr->geometry().width(), scaled.height());
-                                x = scr->geometry().x();
-                                y = scr->geometry().y();
-                           }
-                        }
-                        else  { // w_ratio > h_ratio
-                            if(wallpaperMode_ == WallpaperFit) {
-                                // fit vertically
-                                scaled = image.scaledToHeight(scr->geometry().height(), Qt::SmoothTransformation);
-                                x = scr->geometry().x() + (scr->geometry().width() - scaled.width()) / 2;
-                                y = scr->geometry().y();
+                    else if((wallpaperMode_ == WallpaperFit || wallpaperMode_ == WallpaperZoom)
+                            && image.width() > 0 && image.height() > 0) {
+                        for(const auto& scr : screens) {
+                            // get the screen-to-image ratio to calculate the scale factors
+                            const qreal w_ratio = static_cast<qreal>(scr->geometry().width()) / image.width();
+                            const qreal h_ratio = static_cast<qreal>(scr->geometry().height()) / image.height();
+                            if(w_ratio <= h_ratio) {
+                                if(wallpaperMode_ == WallpaperFit) {
+                                    // fit horizontally
+                                    scaled = image.scaledToWidth(scr->geometry().width(), Qt::SmoothTransformation);
+                                    x = scr->geometry().x();
+                                    y = scr->geometry().y() + (scr->geometry().height() - scaled.height()) / 2;
+                                }
+                                else { // zoom
+                                    // fit vertically
+                                    scaled = image.scaledToHeight(scr->geometry().height(), Qt::SmoothTransformation);
+                                    // crop to avoid overlapping
+                                    int x_gap = (scaled.width() - scr->geometry().width()) / 2;
+                                    scaled = scaled.copy(x_gap, 0, scr->geometry().width(), scaled.height());
+                                    x = scr->geometry().x();
+                                    y = scr->geometry().y();
+                                }
                             }
-                            else { // zoom
-                                // fit horizonatally
-                                scaled = image.scaledToWidth(scr->geometry().width(), Qt::SmoothTransformation);
-                                // crop to avoid overlapping
-                                int y_gap = (scaled.height() - scr->geometry().height()) / 2;
-                                scaled = scaled.copy(0, y_gap, scaled.width(), scr->geometry().height());
-                                x = scr->geometry().x();
-                                y = scr->geometry().y();
+                            else  { // w_ratio > h_ratio
+                                if(wallpaperMode_ == WallpaperFit) {
+                                    // fit vertically
+                                    scaled = image.scaledToHeight(scr->geometry().height(), Qt::SmoothTransformation);
+                                    x = scr->geometry().x() + (scr->geometry().width() - scaled.width()) / 2;
+                                    y = scr->geometry().y();
+                                }
+                                else { // zoom
+                                    // fit horizonatally
+                                    scaled = image.scaledToWidth(scr->geometry().width(), Qt::SmoothTransformation);
+                                    // crop to avoid overlapping
+                                    int y_gap = (scaled.height() - scr->geometry().height()) / 2;
+                                    scaled = scaled.copy(0, y_gap, scaled.width(), scr->geometry().height());
+                                    x = scr->geometry().x();
+                                    y = scr->geometry().y();
+                                }
                             }
+                            painter.drawImage(x, y, scaled);
                         }
-                        painter.drawImage(x, y, scaled);
                     }
                 }
             }
@@ -803,8 +817,13 @@ bool DesktopWindow::pickWallpaper() {
                        files.removeOne(ls); // choose from other images
                }
                // this is needed for the randomness, especially when choosing the first wallpaper
-               qsrand((uint)QTime::currentTime().msec());
-               int randomValue = qrand() % files.size();
+
+               // Conversion notes:
+               // The random generated value can be safely converted to an int because it's max
+               // value is also an int (files.size() - 1).
+               // files.size() - 1 (the highest random value) is always => 1
+               // files.at(0) -> valid, files.at(size) -> invalid
+               int randomValue = static_cast<int>(QRandomGenerator::global()->bounded(static_cast<quint32>(files.size())));
                wallpaperFile_ = dir + files.at(randomValue);
            }
            else {
@@ -856,6 +875,7 @@ void DesktopWindow::updateFromSettings(Settings& settings, bool changeSlide) {
     setForeground(settings.desktopFgColor());
     setBackground(settings.desktopBgColor());
     setShadow(settings.desktopShadowColor());
+    fileLauncher_.setOpenWithDefaultFileManager(settings.openWithDefaultFileManager());
     desktopHideItems_ = settings.desktopHideItems();
     if(desktopHideItems_) {
         // hide all items by hiding the list view and also
@@ -909,8 +929,7 @@ void DesktopWindow::onFileClicked(int type, const std::shared_ptr<const Fm::File
     }
     else {
         // special right-click menus for our desktop shortcuts
-        if(fileInfo && (fileInfo->isDesktopEntry() || fileInfo->isShortcut())
-           && type == Fm::FolderView::ContextMenuClick) {
+        if(fileInfo && fileInfo->isDesktopEntry() && type == Fm::FolderView::ContextMenuClick) {
             Settings& settings = static_cast<Application* >(qApp)->settings();
             const QStringList ds = settings.desktopShortcuts();
             if(!ds.isEmpty()) {
@@ -1090,33 +1109,6 @@ void DesktopWindow::onModelSortFilterChanged() {
     settings.setDesktopSortHiddenLast(proxyModel_->hiddenLast());
 }
 
-void DesktopWindow::onDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight) {
-    /****************************************************************************
-     NOTE: The display names of desktop entries and shortcuts may change without
-     their files being renamed and, on such occasions, a relayout will be needed.
-     Since there is no signal for that, we use the signal dataChanged() and the
-     QHash displayNames_, which remembers such display names with every relayout.
-     ****************************************************************************/
-    if(topLeft.column() == 0) {
-        bool relayout(false);
-        for(int i = topLeft.row(); i <= bottomRight.row(); ++i) {
-            QModelIndex index = topLeft.sibling(i, 0);
-            if(index.isValid() && displayNames_.contains(index)) {
-                auto file = proxyModel_->fileInfoFromIndex(index);
-                if(displayNames_[index] != file->displayName()) {
-                    relayout = true;
-                    break;
-                }
-            }
-        }
-        if(relayout) {
-            queueRelayout();
-            // parts of the old display name might still be visible if it's long
-            listView_->viewport()->update();
-        }
-    }
-}
-
 void DesktopWindow::onFolderStartLoading() { // desktop may be reloaded
     if(model_) {
         disconnect(model_, &Fm::FolderModel::filesAdded, this, &DesktopWindow::onFilesAdded);
@@ -1218,15 +1210,21 @@ void DesktopWindow::trustOurDesktopShortcut(std::shared_ptr<const Fm::FileInfo>
         return;
     }
     const QString fileName = QString::fromStdString(file->name());
-    auto homeExec = Fm::CStrPtr(g_strconcat("pcmanfm-qt ", Fm::FilePath::homeDir().toString().get(), nullptr));
-    const char* execStr = fileName == QLatin1String("trash-can.desktop") && ds.contains(QLatin1String("Trash")) ? "pcmanfm-qt trash:///" :
-                          fileName == QLatin1String("user-home.desktop") && ds.contains(QLatin1String("Home")) ? homeExec.get() :
+    bool isHome(fileName == QLatin1String("user-home.desktop") && ds.contains(QLatin1String("Home")));
+    // we need this in advance because we get execStr with a nested ternary operator below
+    auto homeExec = isHome && settings.openWithDefaultFileManager()
+                        ? Fm::FilePath::homeDir().toString()
+                        : Fm::CStrPtr(g_strconcat("pcmanfm-qt ", Fm::FilePath::homeDir().toString().get(), nullptr));
+    const char* execStr = isHome ? homeExec.get() :
+                          fileName == QLatin1String("trash-can.desktop") && ds.contains(QLatin1String("Trash")) ? "pcmanfm-qt trash:///" :
                           fileName == QLatin1String("computer.desktop") && ds.contains(QLatin1String("Computer")) ? "pcmanfm-qt computer:///" :
                           fileName == QLatin1String("network.desktop") && ds.contains(QLatin1String("Network")) ? "pcmanfm-qt network:///" : nullptr;
     if(execStr) {
         GKeyFile* kf = g_key_file_new();
         if(g_key_file_load_from_file(kf, file->path().toString().get(), G_KEY_FILE_NONE, nullptr)) {
-            Fm::CStrPtr str{g_key_file_get_string(kf, "Desktop Entry", "Exec", nullptr)};
+            Fm::CStrPtr str{g_key_file_get_string(kf, "Desktop Entry",
+                                                  isHome && settings.openWithDefaultFileManager() ? "URL" : "Exec",
+                                                  nullptr)};
             if(str && strcmp(str.get(), execStr) == 0) {
                 file->setTrustable(true);
             }
@@ -1242,7 +1240,6 @@ void DesktopWindow::relayoutItems() {
     if(screen == nullptr) {
         return;
     }
-    displayNames_.clear();
     loadItemPositions(); // something may have changed
     // qDebug("relayoutItems()");
     if(relayoutTimer_) {
@@ -1271,9 +1268,7 @@ void DesktopWindow::relayoutItems() {
             file->setTrustable(true);
             filesToTrust_.removeAll(QString::fromStdString(name));
         }
-        if(file->isDesktopEntry() || file->isShortcut()) {
-            // remember display names of desktop entries and shortcuts
-            displayNames_[index] = file->displayName();
+        else if(file->isDesktopEntry()) {
             trustOurDesktopShortcut(file);
          }
         auto find_it = customItemPos_.find(name);
@@ -1887,9 +1882,9 @@ QModelIndex DesktopWindow::indexForPos(bool* isTrash, const QPoint& pos, const Q
     p.setY(qMax(workArea.top(), p.y()));
     alignToGrid(p, workArea.topLeft(), grid, listView_->spacing());
     // then put the point where the icon rectangle may be
-    // (if there is any item, its icon is immediately below the middle of the top side)
+    // (if there is any item, its icon is immediately below the middle of its top side)
     p.setX(p.x() + (grid.width() + listView_->spacing()) / 2);
-    p.setY(p.y() + listView_->spacing() / 2 + 1);
+    p.setY(p.y() + listView_->spacing() / 2 + getMargins().height() + 1);
     // if this is the Trash cell, return its index
     QModelIndex indx = listView_->indexAt(p);
     if(indx.isValid() && indx.model()) {
