@@ -45,7 +45,6 @@
 #include <libfm-qt/pathedit.h>
 #include <libfm-qt/pathbar.h>
 #include <libfm-qt/core/fileinfo.h>
-#include <libfm-qt/mountoperation.h>
 #include "ui_about.h"
 #include "ui_shortcuts.h"
 #include "application.h"
@@ -660,9 +659,21 @@ int MainWindow::addTabWithPage(TabPage* page, ViewFrame* viewFrame, Fm::FilePath
     if(path) {
         page->chdir(path, true);
     }
-    viewFrame->getTabBar()->insertTab(index, page->windowTitle());
+
+    QString tabText = page->title();
+    // remove newline (not all styles can handle it) and distinguish ampersand from mnemonic
+    tabText.replace(QLatin1Char('\n'), QLatin1Char(' '))
+           .replace(QLatin1Char('&'), QLatin1String("&&"));
+    viewFrame->getTabBar()->insertTab(index, tabText);
 
     Settings& settings = static_cast<Application*>(qApp)->settings();
+    if(settings.switchToNewTab()) {
+        viewFrame->getTabBar()->setCurrentIndex(index);
+        if (isMinimized()) {
+            setWindowState((windowState() & ~Qt::WindowMinimized) | Qt::WindowActive);
+            show();
+        }
+    }
     if(!settings.alwaysShowTabs()) {
         viewFrame->getTabBar()->setVisible(viewFrame->getTabBar()->count() > 1);
     }
@@ -670,9 +681,9 @@ int MainWindow::addTabWithPage(TabPage* page, ViewFrame* viewFrame, Fm::FilePath
 }
 
 // add a new tab
-int MainWindow::addTab(Fm::FilePath path, ViewFrame* viewFrame) {
+void MainWindow::addTab(Fm::FilePath path, ViewFrame* viewFrame) {
     TabPage* newPage = new TabPage(this);
-    return addTabWithPage(newPage, viewFrame, path);
+    addTabWithPage(newPage, viewFrame, path);
 }
 
 void MainWindow::toggleMenuBar(bool /*checked*/) {
@@ -791,8 +802,7 @@ void MainWindow::on_actionGo_triggered() {
 
 void MainWindow::on_actionNewTab_triggered() {
     auto path = currentPage()->path();
-    int index = addTab(path);
-    activeViewFrame_->getTabBar()->setCurrentIndex(index);
+    addTab(path);
 }
 
 void MainWindow::on_actionNewWin_triggered() {
@@ -1026,7 +1036,7 @@ void MainWindow::on_actionAbout_triggered() {
     // the about dialog
     class AboutDialog : public QDialog {
     public:
-        explicit AboutDialog(QWidget* parent = 0, Qt::WindowFlags f = 0) : QDialog(parent, f) {
+        explicit AboutDialog(QWidget* parent = 0, Qt::WindowFlags f = Qt::WindowFlags()) : QDialog(parent, f) {
             ui.setupUi(this);
             ui.version->setText(tr("Version: %1").arg(QStringLiteral(PCMANFM_QT_VERSION)));
         }
@@ -1040,7 +1050,7 @@ void MainWindow::on_actionAbout_triggered() {
 void MainWindow::on_actionHiddenShortcuts_triggered() {
     class HiddenShortcutsDialog : public QDialog {
     public:
-        explicit HiddenShortcutsDialog(QWidget* parent = 0, Qt::WindowFlags f = 0) : QDialog(parent, f) {
+        explicit HiddenShortcutsDialog(QWidget* parent = 0, Qt::WindowFlags f = Qt::WindowFlags()) : QDialog(parent, f) {
             ui.setupUi(this);
             ui.treeWidget->setRootIsDecorated(false);
             ui.treeWidget->header()->setSectionResizeMode(QHeaderView::Stretch);
@@ -1103,10 +1113,14 @@ void MainWindow::onTabBarTabMoved(int from, int to) {
     }
 }
 
+QList<Fm::MountOperation*> MainWindow::pendingMountOperations() const {
+    return ui.sidePane->findChildren<MountOperation*>();
+}
+
 void MainWindow::onFolderUnmounted() {
     TabPage* tabPage = static_cast<TabPage*>(sender());
     if(ViewFrame* viewFrame = viewFrameForTabPage(tabPage)) {
-        const QList<MountOperation*> ops = ui.sidePane->findChildren<MountOperation*>();
+        const QList<MountOperation*> ops = pendingMountOperations();
         if(ops.isEmpty()) { // unmounting is done somewhere else
             Settings& settings = static_cast<Application*>(qApp)->settings();
             if(settings.closeOnUnmount()) {
@@ -1124,7 +1138,7 @@ void MainWindow::onFolderUnmounted() {
         else { // wait for all (un-)mount operations to be finished (otherwise, they might be cancelled)
             for(const MountOperation* op : ops) {
                 connect(op, &QObject::destroyed, tabPage, [this, tabPage, viewFrame] {
-                    if(ui.sidePane->findChildren<MountOperation*>().isEmpty()) {
+                    if(pendingMountOperations().isEmpty()) {
                         Settings& settings = static_cast<Application*>(qApp)->settings();
                         if(settings.closeOnUnmount()) {
                             viewFrame->getStackedWidget()->removeWidget(tabPage);
@@ -1179,6 +1193,23 @@ void MainWindow::closeEvent(QCloseEvent* event) {
             settings.setLastWindowHeight(height());
         }
     }
+
+    // remember last tab paths only if this is the last window
+    QStringList tabPaths;
+    if(lastActive_ == nullptr && settings.reopenLastTabs()) {
+        for(int i = 0; i < ui.viewSplitter->count(); ++i) {
+            if(ViewFrame* viewFrame = qobject_cast<ViewFrame*>(ui.viewSplitter->widget(i))) {
+                int n = viewFrame->getStackedWidget()->count();
+                for(int j = 0; j < n; ++j) {
+                    if(TabPage* page = static_cast<TabPage*>(viewFrame->getStackedWidget()->widget(j))) {
+                        tabPaths.append(QString::fromUtf8(page->path().toString().get()));
+                    }
+                }
+            }
+        }
+        tabPaths.removeDuplicates();
+    }
+    settings.setTabPaths(tabPaths);
 }
 
 void MainWindow::onTabBarCurrentChanged(int index) {
@@ -1328,7 +1359,7 @@ void MainWindow::updateUIForCurrentPage(bool setFocus) {
     TabPage* tabPage = currentPage();
 
     if(tabPage) {
-        setWindowTitle(tabPage->windowTitle());
+        setWindowTitle(tabPage->title());
         if(splitView_) {
             if(Fm::PathBar* pathBar = qobject_cast<Fm::PathBar*>(activeViewFrame_->getTopBar())) {
                 pathBar->setPath(tabPage->path());
@@ -1427,17 +1458,21 @@ void MainWindow::onStackedWidgetWidgetRemoved(int index) {
     }
 }
 
-void MainWindow::onTabPageTitleChanged(QString title) {
+void MainWindow::onTabPageTitleChanged() {
     TabPage* tabPage = static_cast<TabPage*>(sender());
     if(ViewFrame* viewFrame = viewFrameForTabPage(tabPage)) {
         int index = viewFrame->getStackedWidget()->indexOf(tabPage);
         if(index >= 0) {
-            viewFrame->getTabBar()->setTabText(index, title);
+            QString tabText = tabPage->title();
+            // remove newline and distinguish ampersand from mnemonic
+            tabText.replace(QLatin1Char('\n'), QLatin1Char(' '))
+                   .replace(QLatin1Char('&'), QLatin1String("&&"));
+            viewFrame->getTabBar()->setTabText(index, tabText);
         }
 
         if(viewFrame == activeViewFrame_) {
             if(tabPage == currentPage()) {
-                setWindowTitle(title);
+                setWindowTitle(tabPage->title());
 
                 // Since TabPage::titleChanged is emitted on changing directory,
                 // the enabled state of Paste action should be updated here
