@@ -85,6 +85,8 @@ Settings::Settings():
     desktopSortHiddenLast_(false),
     alwaysShowTabs_(true),
     showTabClose_(true),
+    switchToNewTab_(false),
+    reopenLastTabs_(false),
     rememberWindowSize_(true),
     fixedWindowWidth_(640),
     fixedWindowHeight_(480),
@@ -118,19 +120,17 @@ Settings::Settings():
     showThumbnails_(true),
     archiver_(),
     siUnit_(false),
-    placesHome_(true),
-    placesDesktop_(true),
-    placesApplications_(true),
-    placesTrash_(true),
-    placesRoot_(true),
-    placesComputer_(true),
-    placesNetwork_(true),
+    backupAsHidden_(false),
+    showFullNames_(true),
+    shadowHidden_(true),
+    noItemTooltip_(false),
     bigIconSize_(48),
     smallIconSize_(24),
     sidePaneIconSize_(24),
     thumbnailIconSize_(128),
     folderViewCellMargins_(QSize(3, 3)),
     desktopCellMargins_(QSize(3, 1)),
+    openWithDefaultFileManager_(false),
     searchNameCaseInsensitive_(false),
     searchContentCaseInsensitive_(false),
     searchNameRegexp_(true),
@@ -261,6 +261,7 @@ bool Settings::loadFile(QString filePath) {
 
     desktopCellMargins_ = (settings.value(QStringLiteral("DesktopCellMargins"), QSize(3, 1)).toSize()
                            .expandedTo(QSize(0, 0))).boundedTo(QSize(48, 48));
+    openWithDefaultFileManager_ = settings.value(QStringLiteral("OpenWithDefaultFileManager"), false).toBool();
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Volume"));
@@ -289,6 +290,7 @@ bool Settings::loadFile(QString filePath) {
     setBackupAsHidden(settings.value(QStringLiteral("BackupAsHidden"), false).toBool());
     showFullNames_ = settings.value(QStringLiteral("ShowFullNames"), true).toBool();
     shadowHidden_ = settings.value(QStringLiteral("ShadowHidden"), true).toBool();
+    noItemTooltip_ = settings.value(QStringLiteral("NoItemTooltip"), false).toBool();
 
     // override config in libfm's FmConfig
     bigIconSize_ = toIconSize(settings.value(QStringLiteral("BigIconSize"), 48).toInt(), Big);
@@ -306,13 +308,6 @@ bool Settings::loadFile(QString filePath) {
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Places"));
-    placesHome_ = settings.value(QStringLiteral("PlacesHome"), true).toBool();
-    placesDesktop_ = settings.value(QStringLiteral("PlacesDesktop"), true).toBool();
-    placesApplications_ = settings.value(QStringLiteral("PlacesApplications"), true).toBool();
-    placesTrash_ = settings.value(QStringLiteral("PlacesTrash"), true).toBool();
-    placesRoot_ = settings.value(QStringLiteral("PlacesRoot"), true).toBool();
-    placesComputer_ = settings.value(QStringLiteral("PlacesComputer"), true).toBool();
-    placesNetwork_ = settings.value(QStringLiteral("PlacesNetwork"), true).toBool();
     hiddenPlaces_ = settings.value(QStringLiteral("HiddenPlaces")).toStringList().toSet();
     settings.endGroup();
 
@@ -325,6 +320,9 @@ bool Settings::loadFile(QString filePath) {
     rememberWindowSize_ = settings.value(QStringLiteral("RememberWindowSize"), true).toBool();
     alwaysShowTabs_ = settings.value(QStringLiteral("AlwaysShowTabs"), true).toBool();
     showTabClose_ = settings.value(QStringLiteral("ShowTabClose"), true).toBool();
+    switchToNewTab_ = settings.value(QStringLiteral("SwitchToNewTab"), false).toBool();
+    reopenLastTabs_ = settings.value(QStringLiteral("ReopenLastTabs"), false).toBool();
+    tabPaths_ = settings.value(QStringLiteral("TabPaths")).toStringList();
     splitterPos_ = settings.value(QStringLiteral("SplitterPos"), 150).toInt();
     sidePaneVisible_ = settings.value(QStringLiteral("SidePaneVisible"), true).toBool();
     sidePaneMode_ = sidePaneModeFromString(settings.value(QStringLiteral("SidePaneMode")).toString());
@@ -402,6 +400,7 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("SortFolderFirst"), desktopSortFolderFirst_);
     settings.setValue(QStringLiteral("SortHiddenLast"), desktopSortHiddenLast_);
     settings.setValue(QStringLiteral("DesktopCellMargins"), desktopCellMargins_);
+    settings.setValue(QStringLiteral("OpenWithDefaultFileManager"), openWithDefaultFileManager_);
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Volume"));
@@ -430,6 +429,7 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("BackupAsHidden"), backupAsHidden_);
     settings.setValue(QStringLiteral("ShowFullNames"), showFullNames_);
     settings.setValue(QStringLiteral("ShadowHidden"), shadowHidden_);
+    settings.setValue(QStringLiteral("NoItemTooltip"), noItemTooltip_);
 
     // override config in libfm's FmConfig
     settings.setValue(QStringLiteral("BigIconSize"), bigIconSize_);
@@ -447,20 +447,8 @@ bool Settings::saveFile(QString filePath) {
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Places"));
-    settings.setValue(QStringLiteral("PlacesHome"), placesHome_);
-    settings.setValue(QStringLiteral("PlacesDesktop"), placesDesktop_);
-    settings.setValue(QStringLiteral("PlacesApplications"), placesApplications_);
-    settings.setValue(QStringLiteral("PlacesTrash"), placesTrash_);
-    settings.setValue(QStringLiteral("PlacesRoot"), placesRoot_);
-    settings.setValue(QStringLiteral("PlacesComputer"), placesComputer_);
-    settings.setValue(QStringLiteral("PlacesNetwork"), placesNetwork_);
-    if (hiddenPlaces_.isEmpty()) {  // don't save "@Invalid()"
-        settings.remove(QStringLiteral("HiddenPlaces"));
-    }
-    else {
-        QStringList hiddenPlaces = hiddenPlaces_.toList();
-        settings.setValue(QStringLiteral("HiddenPlaces"), hiddenPlaces);
-    }
+    QStringList hiddenPlaces = hiddenPlaces_.toList();
+    settings.setValue(QStringLiteral("HiddenPlaces"), hiddenPlaces);
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Window"));
@@ -472,6 +460,9 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("RememberWindowSize"), rememberWindowSize_);
     settings.setValue(QStringLiteral("AlwaysShowTabs"), alwaysShowTabs_);
     settings.setValue(QStringLiteral("ShowTabClose"), showTabClose_);
+    settings.setValue(QStringLiteral("SwitchToNewTab"), switchToNewTab_);
+    settings.setValue(QStringLiteral("ReopenLastTabs"), reopenLastTabs_);
+    settings.setValue(QStringLiteral("TabPaths"), tabPaths_);
     settings.setValue(QStringLiteral("SplitterPos"), splitterPos_);
     settings.setValue(QStringLiteral("SidePaneVisible"), sidePaneVisible_);
     settings.setValue(QStringLiteral("SidePaneMode"), QString::fromUtf8(sidePaneModeToString(sidePaneMode_)));
