@@ -437,6 +437,30 @@ public:
         showTabClose_ = showTabClose;
     }
 
+    bool switchToNewTab() const {
+        return switchToNewTab_;
+    }
+
+    void setSwitchToNewTab(bool showTabClose) {
+        switchToNewTab_ = showTabClose;
+    }
+
+    bool reopenLastTabs() const {
+        return reopenLastTabs_;
+    }
+
+    void setReopenLastTabs(bool reopenLastTabs) {
+        reopenLastTabs_ = reopenLastTabs;
+    }
+
+    QStringList tabPaths() const {
+        return tabPaths_;
+    }
+
+    void setTabPaths(const QStringList& tabPaths) {
+        tabPaths_ = tabPaths;
+    }
+
     bool rememberWindowSize() const {
         return rememberWindowSize_;
     }
@@ -564,63 +588,6 @@ public:
         sortCaseSensitive_ = value;
     }
 
-
-    bool placesHome() const {
-        return placesHome_;
-    }
-
-    void setPlacesHome(bool placesHome) {
-        placesHome_ = placesHome;
-    }
-
-    bool placesDesktop() const {
-        return placesDesktop_;
-    }
-
-    void setPlacesDesktop(bool placesDesktop) {
-        placesDesktop_ = placesDesktop;
-    }
-
-    bool placesApplications() const {
-        return placesApplications_;
-    }
-
-    void setPlacesApplications(bool placesApplications) {
-        placesApplications_ = placesApplications;
-    }
-
-    bool placesTrash() const {
-        return placesTrash_;
-    }
-
-    void setPlacesTrash(bool placesTrash) {
-        placesTrash_ = placesTrash;
-    }
-
-    bool placesRoot() const {
-        return placesRoot_;
-    }
-
-    void setPlacesRoot(bool placesRoot) {
-        placesRoot_ = placesRoot;
-    }
-
-    bool placesComputer() const {
-        return placesComputer_;
-    }
-
-    void setPlacesComputer(bool placesComputer) {
-        placesComputer_ = placesComputer;
-    }
-
-    bool placesNetwork() const {
-        return placesNetwork_;
-    }
-
-    void setPlacesNetwork(bool placesNetwork) {
-        placesNetwork_ = placesNetwork;
-    }
-
     QSet<QString> getHiddenPlaces() const {
         return hiddenPlaces_;
     }
@@ -634,7 +601,6 @@ public:
         }
     }
 
-
     Qt::SortOrder sortOrder() const {
         return sortOrder_;
     }
@@ -808,6 +774,13 @@ public:
         desktopCellMargins_ = size;
     }
 
+    bool openWithDefaultFileManager() const {
+        return openWithDefaultFileManager_;
+    }
+
+    void setOpenWithDefaultFileManager(bool open) {
+        openWithDefaultFileManager_ = open;
+    }
 
     bool showThumbnails() {
         return showThumbnails_;
@@ -872,6 +845,14 @@ public:
         shadowHidden_ = value;
     }
 
+    bool noItemTooltip() const {
+        return noItemTooltip_;
+    }
+
+    void setNoItemTooltip(bool noTooltip) {
+        noItemTooltip_ = noTooltip;
+    }
+
     bool onlyUserTemplates() const {
         return onlyUserTemplates_;
     }
@@ -1029,6 +1010,9 @@ private:
 
     bool alwaysShowTabs_;
     bool showTabClose_;
+    bool switchToNewTab_;
+    bool reopenLastTabs_;
+    QStringList tabPaths_;
     bool rememberWindowSize_;
     int fixedWindowWidth_;
     int fixedWindowHeight_;
@@ -1069,14 +1053,8 @@ private:
     bool backupAsHidden_;
     bool showFullNames_;
     bool shadowHidden_;
+    bool noItemTooltip_;
 
-    bool placesHome_;
-    bool placesDesktop_;
-    bool placesApplications_;
-    bool placesTrash_;
-    bool placesRoot_;
-    bool placesComputer_;
-    bool placesNetwork_;
     QSet<QString> hiddenPlaces_;
 
     int bigIconSize_;
@@ -1091,6 +1069,8 @@ private:
     QSize folderViewCellMargins_;
     QSize desktopCellMargins_;
 
+    bool openWithDefaultFileManager_;
+
     // search settings
     bool searchNameCaseInsensitive_;
     bool searchContentCaseInsensitive_;
