@@ -36,7 +36,6 @@
 #include "settings.h"
 #include "application.h"
 #include <QTimer>
-#include <QTextStream>
 #include <QDebug>
 
 using namespace Fm;
@@ -149,6 +148,10 @@ TabPage::TabPage(QWidget* parent):
     folderView_->setModel(proxyModel_);
     verticalLayout->addWidget(folderView_);
 
+    if(settings.noItemTooltip()) {
+        folderView_->childView()->viewport()->installEventFilter(this);
+    }
+
     // filter-bar and its settings
     filterBar_ = new FilterBar();
     verticalLayout->addWidget(filterBar_);
@@ -212,11 +215,14 @@ void TabPage::showFilterBar() {
 
 bool TabPage::eventFilter(QObject* watched, QEvent* event) {
     // when a text is typed inside the view, type it inside the filter-bar
-    if(filterBar_ && watched == folderView_->childView() &&  event->type() == QEvent::KeyPress) {
+    if(filterBar_ && watched == folderView_->childView() && event->type() == QEvent::KeyPress) {
         if(QKeyEvent* ke = static_cast<QKeyEvent*>(event)) {
             filterBar_->keyPressed(ke);
         }
     }
+    else if (watched == folderView_->childView()->viewport() && event->type() == QEvent::ToolTip) {
+        return true;
+    }
     return QWidget::eventFilter(watched, event);
 }
 
@@ -307,9 +313,7 @@ void TabPage::onFolderStartLoading() {
 }
 
 void TabPage::onUiUpdated() {
-    // scroll to recorded position
-    folderView_->childView()->verticalScrollBar()->setValue(browseHistory().currentScrollPos());
-
+    bool scrolled = false;
     // if the current folder is the parent folder of the last browsed folder,
     // select the folder item in current view.
     if(lastFolderPath_ && lastFolderPath_.parent() == path()) {
@@ -317,13 +321,17 @@ void TabPage::onUiUpdated() {
         if(index.isValid()) {
             folderView_->childView()->scrollTo(index, QAbstractItemView::EnsureVisible);
             folderView_->childView()->setCurrentIndex(index);
+            scrolled = true;
         }
     }
-    else { // set the first item as current
+    if(!scrolled) {
+        // set the first item as current
         QModelIndex firstIndx = proxyModel_->index(0, 0);
         if (firstIndx.isValid()) {
             folderView_->selectionModel()->setCurrentIndex(firstIndx, QItemSelectionModel::NoUpdate);
         }
+        // scroll to recorded position
+        folderView_->childView()->verticalScrollBar()->setValue(browseHistory().currentScrollPos());
     }
 
     if(folderModel_) {
@@ -332,6 +340,21 @@ void TabPage::onUiUpdated() {
         // get ready to select files that may be added later
         connect(folderModel_, &Fm::FolderModel::filesAdded, this, &TabPage::onFilesAdded);
     }
+
+    // in the single-click mode, set the cursor shape of the view to a pointing hand
+    // only if there is an item under it
+    if(folderView_->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick)) {
+        QTimer::singleShot(0, folderView_, [this] {
+            QPoint pos = folderView_->childView()->mapFromGlobal(QCursor::pos());
+            QModelIndex index = folderView_->childView()->indexAt(pos);
+            if(index.isValid()) {
+                folderView_->setCursor(Qt::PointingHandCursor);
+            }
+            else {
+                folderView_->setCursor(Qt::ArrowCursor);
+            }
+        });
+    }
 }
 
 void TabPage::onFileSizeChanged(const QModelIndex& index) {
@@ -369,8 +392,8 @@ void TabPage::onFilesAdded(Fm::FileInfoList files) {
 void TabPage::onFolderFinishLoading() {
     auto fi = folder_->info();
     if(fi) { // if loading of the folder fails, it's possible that we don't have FmFileInfo.
-        setWindowTitle(fi->displayName());
-        Q_EMIT titleChanged(fi->displayName());
+        title_ = fi->displayName();
+        Q_EMIT titleChanged();
     }
 
     folder_->queryFilesystemInfo(); // FIXME: is this needed?
@@ -477,9 +500,8 @@ QString TabPage::formatStatusText() {
         }
         auto fi = folder_->info();
         if (fi && fi->isSymlink()) {
-            text += QStringLiteral(" %2(%1)")
-                    .arg(encloseWithBidiMarks(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target())),
-                    (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
+            text += QStringLiteral(" (%1)")
+                    .arg(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target()));
         }
         return text;
     }
@@ -509,7 +531,14 @@ void TabPage::onFolderUnmount() {
     // because unmounting might be done from places view, in which case,
     // the mount operation is a child of the places view and should be
     // finished before doing anything else.
-    freeFolder();
+    if(static_cast<Application*>(qApp)->settings().closeOnUnmount()) {
+        freeFolder();
+    }
+    else if (folder_) {
+        // the folder shouldn't be freed here because the dir will be changed by
+        // the slot of MainWindow but it should be disconnected from all signals
+        disconnect(folder_.get(), nullptr, this, nullptr);
+    }
     Q_EMIT folderUnmounted();
 }
 
@@ -562,7 +591,9 @@ void TabPage::chdir(Fm::FilePath newPath, bool addHistory) {
         freeFolder();
     }
 
-    Q_EMIT titleChanged(QString::fromUtf8(newPath.baseName().get()));  // FIXME: display name
+    // set title as with path button (will change if the new folder is loaded)
+    title_ = QString::fromUtf8(newPath.baseName().get());
+    Q_EMIT titleChanged();
 
     folder_ = Fm::Folder::fromPath(newPath);
     if(addHistory) {
@@ -639,17 +670,6 @@ void TabPage::reload() {
     }
 }
 
-// 200e LEFT-TO-RIGHT MARK
-// 200f RIGHT-TO-LEFT MARK
-// 202a LEFT-TO-RIGHT EMBEDDING
-// 202b RIGHT-TO-LEFT EMBEDDING
-// 202c POP DIRECTIONAL FORMATTING
-QString TabPage::encloseWithBidiMarks(const QString& text) {
-    QChar bidiMark = text.isRightToLeft() ? QChar(0x200f) : QChar(0x200e);
-    QChar embedBidiMark = text.isRightToLeft() ? QChar(0x202b) : QChar(0x202a);
-    return embedBidiMark+text+bidiMark+QChar(0x202c);
-}
-
 // when the current selection in the folder view is changed
 void TabPage::onSelChanged() {
     QString msg;
@@ -665,34 +685,30 @@ void TabPage::onSelChanged() {
                                    ? QString::fromStdString(fi->name())
                                    : fi->displayName();
                 if(fi->isSymlink()) {
-                    msg = QStringLiteral("%5\"%1\" %5(%2) %5%3 %5(%4)")
-                          .arg(encloseWithBidiMarks(name),
-                          encloseWithBidiMarks(Fm::formatFileSize(fi->size(), fm_config->si_unit)),
-                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
-                          encloseWithBidiMarks(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target())),
-                          (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
+                    msg = QStringLiteral("\"%1\" (%2) %3 (%4)")
+                          .arg(name,
+                          Fm::formatFileSize(fi->size(), fm_config->si_unit),
+                          QString::fromUtf8(fi->mimeType()->desc()),
+                          tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target()));
                 }
                 else {
-                    msg = QStringLiteral("%4\"%1\" %4(%2) %4%3")
-                          .arg(encloseWithBidiMarks(name),
-                          encloseWithBidiMarks(Fm::formatFileSize(fi->size(), fm_config->si_unit)), // FIXME: deprecate fm_config
-                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
-                          (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
+                    msg = QStringLiteral("\"%1\" (%2) %3")
+                          .arg(name,
+                          Fm::formatFileSize(fi->size(), fm_config->si_unit), // FIXME: deprecate fm_config
+                          QString::fromUtf8(fi->mimeType()->desc()));
                 }
             }
             else {
                 if(fi->isSymlink()) {
-                    msg = QStringLiteral("%4\"%1\" %4%2 %4(%3)")
-                          .arg(encloseWithBidiMarks(fi->displayName()),
-                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
-                          encloseWithBidiMarks(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target())),
-                          (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
+                    msg = QStringLiteral("\"%1\" %2 (%3)")
+                          .arg(fi->displayName(),
+                          QString::fromUtf8(fi->mimeType()->desc()),
+                          tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target()));
                 }
                 else {
-                    msg = QStringLiteral("%3\"%1\" %3%2")
-                          .arg(encloseWithBidiMarks(fi->displayName()),
-                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
-                          (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
+                    msg = QStringLiteral("\"%1\" %2")
+                          .arg(fi->displayName(),
+                          QString::fromUtf8(fi->mimeType()->desc()));
                 }
             }
             /* FIXME: should we support statusbar plugins as in the gtk+ version? */
@@ -775,6 +791,13 @@ void TabPage::up() {
 
 void TabPage::updateFromSettings(Settings& settings) {
     folderView_->updateFromSettings(settings);
+    if(settings.noItemTooltip()) {
+        folderView_->childView()->viewport()->removeEventFilter(this);
+        folderView_->childView()->viewport()->installEventFilter(this);
+    }
+    else {
+        folderView_->childView()->viewport()->removeEventFilter(this);
+    }
 }
 
 void TabPage::setViewMode(Fm::FolderView::ViewMode mode) {
@@ -787,7 +810,9 @@ void TabPage::setViewMode(Fm::FolderView::ViewMode mode) {
     }
     Fm::FolderView::ViewMode prevMode = folderView_->viewMode();
     folderView_->setViewMode(mode);
-    folderView_->childView()->setFocus();
+    if(folderView_->isVisible()) { // in the current tab
+        folderView_->childView()->setFocus();
+    }
     if(prevMode != folderView_->viewMode()) {
         // FolderView::setViewMode() may delete the view to switch between list and tree.
         // So, the event filter should be re-installed and the status message should be updated.
@@ -795,6 +820,10 @@ void TabPage::setViewMode(Fm::FolderView::ViewMode mode) {
             folderView_->childView()->removeEventFilter(this);
             folderView_->childView()->installEventFilter(this);
         }
+        if(settings.noItemTooltip()) {
+            folderView_->childView()->viewport()->removeEventFilter(this);
+            folderView_->childView()->viewport()->installEventFilter(this);
+        }
         onSelChanged();
     }
 }
@@ -866,20 +895,39 @@ void TabPage::applyFilter() {
 
     proxyModel_->updateFilters();
 
+    QModelIndex curIndex = folderView_->selectionModel()->currentIndex();
     QModelIndex firstIndx = proxyModel_->index(0, 0);
-    if (firstIndx.isValid()) {
-        if(proxyFilter_->getFilterStr().isEmpty()) {
-            // if the filter is removed and there is no current item, set the first item as current
-            if (!folderView_->selectionModel()->currentIndex().isValid()) {
-                folderView_->selectionModel()->setCurrentIndex(firstIndx, QItemSelectionModel::NoUpdate);
-            }
+    if(proxyFilter_->getFilterStr().isEmpty()) {
+        if (curIndex.isValid()) {
+            // scroll to the current item on removing filter
+            folderView_->childView()->scrollTo(curIndex, QAbstractItemView::EnsureVisible);
         }
-        else if (!folderView_->hasSelection()) {
-            // if there is a filter but no selection, select the first item
+        else if (firstIndx.isValid()) {
+            // if the filter is removed and there is no current item, set the first item as current
+            folderView_->selectionModel()->setCurrentIndex(firstIndx, QItemSelectionModel::NoUpdate);
+        }
+    }
+    else if (!folderView_->hasSelection()) {
+        // if there is a filter but no selection, select the first item
+        if(firstIndx.isValid()) {
             folderView_->childView()->setCurrentIndex(firstIndx);
             selectFirst = true;
         }
     }
+    else {
+        // if there is a filter and a selection, ensure the current index is visible
+        if (curIndex.isValid()) {
+            folderView_->childView()->scrollTo(curIndex, QAbstractItemView::EnsureVisible);
+        }
+        else { // should never happen
+            QModelIndexList selIndexes = folderView_->selectionModel()->selectedIndexes();
+            if(!selIndexes.isEmpty()) { // always true
+                QModelIndex lastIndex = selIndexes.last();
+                folderView_->selectionModel()->setCurrentIndex(lastIndex, QItemSelectionModel::NoUpdate);
+                folderView_->childView()->scrollTo(lastIndex, QAbstractItemView::EnsureVisible);
+            }
+        }
+    }
 
     // if some selected files are filtered out, "View::selChanged()" won't be emitted
     if(!selectFirst && prevSelSize > folderView_->selectionModel()->selectedIndexes().size()) {
