@@ -199,6 +199,10 @@ public:
 
     void reload();
 
+    QString title() const {
+        return title_;
+    }
+
     QString statusText(StatusTextType type = StatusTextNormal) const {
         return statusText_[type];
     }
@@ -268,7 +272,7 @@ public:
 
 Q_SIGNALS:
     void statusChanged(int type, QString statusText);
-    void titleChanged(QString title);
+    void titleChanged();
     void sortFilterChanged();
     void forwardRequested();
     void backwardRequested();
@@ -289,9 +293,6 @@ private:
     void freeFolder();
     QString formatStatusText();
 
-    // Adds bidi marks (RLM/LRM/RLE/LRE/POP) around the text for the statusbar.
-    QString encloseWithBidiMarks(const QString& text);
-
     void onFolderStartLoading();
     void onFolderFinishLoading();
 
@@ -310,6 +311,7 @@ private:
     ProxyFilter* proxyFilter_;
     QVBoxLayout* verticalLayout;
     std::shared_ptr<Fm::Folder> folder_;
+    QString title_;
     QString statusText_[StatusTextNum];
     Fm::BrowseHistory history_; // browsing history
     Fm::FilePath lastFolderPath_; // last browsed folder
